import { Request, Response } from 'express';
import { AppError } from '../utils/AppError';
import asyncTryCatch from '../utils/AsyncTryCatch';
import UserService from '../services/UserService';

class UserController {
    private userService: UserService;

    constructor() {
        this.userService = new UserService();
    }

    login = asyncTryCatch(async (req: Request, res: Response) => {
        const { email, password } = req.body;
        if (!email || !password) throw new AppError(400, 'ClientError', 'Email and password are required');
        const result = await this.userService.login(email, password);
        if (!result) throw new AppError(401, 'ClientError', 'Invalid email or password');
        res.status(200).json({ email, role: result.role, token: result.token, phoneNumberId: result.phoneNumberId });
    });



    register = asyncTryCatch(async (req: Request, res: Response) => {
        const { name, email, phoneNumberId, role, isActive = true, password } = req.body;
        if (!email || !password || !role || !phoneNumberId) throw new AppError(400, 'ClientError', 'All fields are required');
        await this.userService.createUser(name, email, phoneNumberId, role, isActive, password);
        
        res.status(201).json({name, email, phoneNumberId, role, isActive});
    });
}

export default UserController;
