// In Aysnc funcions errors don't automatically propagate to the global error handler
// unless you manully send it by calling next
// asyncErrorCatcher returns a function so express will pass req, res, next to the function
// we will pass it on to whatven orginalFunction is

import { NextFunction, Request, Response } from 'express';
import { AppError } from './AppError';

export default function asyncTryCatch(
    orginalFunction: Function,
    {
        defaultErrorCode = 500,
        defaultErrorName = 'ServerError',
        defaultErrorMessage = 'An unexpected error occurred!'
    }: {
        defaultErrorCode?: number;
        defaultErrorName?: 'ClientError' | 'ServerError';
        defaultErrorMessage?: string;
    } = {}
) {
    return function (req: Request, res: Response, next: NextFunction) {
        orginalFunction(req, res, next).catch((err: any) => {
            if (!(err instanceof AppError)) {
                const appError = new AppError(defaultErrorCode, defaultErrorName, defaultErrorMessage, err);

                // Attach the original error's stack trace to the AppError object
                // appError.stack = err.stack;
                return next(appError);
            }
            return next(err);
        });
    };
}
