const toSnakeCase = (str: string): string => {
    return str.replace(/[A-Z]/g, (match) => `_${match.toLowerCase()}`);
};

// Type for an object with string keys and any values
type AnyObject = { [key: string]: any };

// Type for an array that could contain any type of value
type AnyArray = any[];

// Recursively convert all keys of an object to snake_case
const convertToSnakeCase = (obj: AnyObject | AnyArray): AnyObject | AnyArray => {
    if (Array.isArray(obj)) {
        // If it's an array, recursively apply to each item
        return obj.map(convertToSnakeCase);
    }

    if (typeof obj === 'object' && obj !== null) {
        // If it's an object, process its keys recursively
        return Object.keys(obj).reduce((acc: AnyObject, key: string) => {
            const snakeKey = toSnakeCase(key); // Convert the key to snake_case
            acc[snakeKey] = convertToSnakeCase(obj[key]); // Recursively convert the value
            return acc;
        }, {} as AnyObject);
    }

    return obj; // Return primitive values as they are
};

export { convertToSnakeCase };
